print("import: 'gnuradio.analog'")
import gnuradio.analog

print("import: 'gnuradio.audio'")
import gnuradio.audio

print("import: 'gnuradio.blocks'")
import gnuradio.blocks

print("import: 'gnuradio.channels'")
import gnuradio.channels

print("import: 'gnuradio.digital'")
import gnuradio.digital

print("import: 'gnuradio.dtv'")
import gnuradio.dtv

print("import: 'gnuradio.fec'")
import gnuradio.fec

print("import: 'gnuradio.fft'")
import gnuradio.fft

print("import: 'gnuradio.filter'")
import gnuradio.filter

print("import: 'gnuradio.gr'")
import gnuradio.gr

print("import: 'gnuradio.network'")
import gnuradio.network

print("import: 'gnuradio.trellis'")
import gnuradio.trellis

print("import: 'gnuradio.vocoder'")
import gnuradio.vocoder

print("import: 'gnuradio.wavelet'")
import gnuradio.wavelet

print("import: 'pmt'")
import pmt

