



gnuradio-config-info -v --prefix --sysconfdir --prefsdir --userprefsdir --prefs --builddate --enabled-components --cc --cxx --cflags
IF %ERRORLEVEL% NEQ 0 exit /B 1
gr_modtool --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Scripts\\gr_filter_design.exe exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Scripts\\gr_modtool.exe exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
